@@ -832,7 +832,7 @@ sectionHeader classes ident level lst = do
                           5  -> "subparagraph"
                           _  -> ""
   inQuote <- gets stInQuote
-  let prefix = if inQuote && level' >= 4
+  let prefix = if inQuote
                   then text "\\mbox{}%"
                   -- needed for \paragraph, \subparagraph in quote environment
                   -- see http://tex.stackexchange.com/questions/169830/
@@ -1375,7 +1375,7 @@ ldfLanguages =
 
 isMathEnv :: Text -> Bool
 isMathEnv t =
-  case T.stripPrefix "\\begin{" t of
+  case T.stripPrefix "\\begin{" (T.dropWhile isSpaceChar t) of
     Nothing -> False
     Just t' -> T.takeWhile (/= '}') t' `elem`
       [ "align", "align*"
@@ -1386,12 +1386,17 @@ isMathEnv t =
       , "darray", "darray*"
       , "gather", "gather*"
       , "multline", "multline*"
-      , "split"
       , "subequations"
       , "equation", "equation*"
       , "eqnarray"
       , "displaymath"
       ]
+ where
+   isSpaceChar '\n' = True
+   isSpaceChar '\r' = True
+   isSpaceChar '\t' = True
+   isSpaceChar ' ' = True
+   isSpaceChar _ = False
 
 -- True if the math needs the cancel package
 needsCancel :: Text -> Bool
