@@ -54,6 +54,7 @@ data WriterState =
   , stLang          :: Maybe Lang    -- ^ lang specified in metadata
   , stInSoulCommand :: Bool          -- ^ in a soul command like ul
   , stCancel        :: Bool          -- ^ true if document uses \cancel
+  , stInCaption     :: Bool          -- ^ true if in a caption
   }
 
 startingState :: WriterOptions -> WriterState
@@ -95,4 +96,5 @@ startingState options =
   , stLang = Nothing
   , stInSoulCommand = False
   , stCancel = False
+  , stInCaption = False
   }
