@@ -50,6 +50,11 @@ module Text.Pandoc.Writers.Shared (
                      , isOrderedListMarker
                      , toTaskListItem
                      , delimited
+                     , allRowsEmpty
+                     , tableBodiesToRows
+                     , insertCurrentSpansAtColumn
+                     , takePreviousSpansAtColumn
+                     , decrementTrailingRowSpans
                      )
 where
 import Safe (lastMay, maximumMay)
@@ -59,7 +64,7 @@ import Data.Either (isRight)
 import Data.Aeson (ToJSON (..), encode)
 import Data.Char (chr, ord, isSpace, isLetter, isUpper)
 import Data.List (groupBy, intersperse, foldl', transpose)
-import Data.List.NonEmpty (NonEmpty(..))
+import Data.List.NonEmpty (NonEmpty((:|)))
 import Data.Text.Conversions (FromText(..))
 import qualified Data.Map as M
 import qualified Data.Text as T
@@ -857,3 +862,66 @@ delimited opener closer content =
   toList (Concat (Concat a b) c) = toList (Concat a (Concat b c))
   toList (Concat a b) = a : toList b
   toList x = [x]
+
+-- | Determine whether all rows and their cells are empty.
+allRowsEmpty :: [Row] -> Bool
+allRowsEmpty = all isEmptyRow
+ where
+  isEmptyRow (Row _ cells) = all isEmptyCell cells
+  isEmptyCell (Cell _ _ _ _ blocks) = null blocks
+
+-- | Concatenates the header and body Rows of a List of TableBody into a flat
+-- List of Rows.
+tableBodiesToRows :: [TableBody] -> [Row]
+tableBodiesToRows = concatMap tableBodyToRows
+  where
+    tableBodyToRows (TableBody _ _ headerRows bodyRows) = headerRows ++ bodyRows
+
+-- | Insert the current span information of a table cell to keep track of it in
+-- subsequent rows.
+--
+-- If 'RowSpan' @> 1@, the current span information will be inserted. Otherwise
+-- the previous span information will be left unchanged.
+--
+-- Use 'takePreviousSpansAtColumn' to take previous span information at
+-- subsequent rows. Use 'decrementTrailingRowSpans' to handle previous trailing
+-- spans at the end of a row.
+--
+-- For writers that need to manually apply the 'RowSpan' of cells over multiple
+-- rows or otherwise have to keep track of it.
+insertCurrentSpansAtColumn :: Int -> M.Map Int (RowSpan, ColSpan) -> RowSpan -> ColSpan -> M.Map Int (RowSpan, ColSpan)
+insertCurrentSpansAtColumn columnPosition previousSpans (RowSpan rowSpan) colSpan =
+  if (rowSpan > 1)
+    then M.insert columnPosition (RowSpan rowSpan - 1, colSpan) previousSpans -- Minus its own row.
+    else previousSpans
+
+-- | Take previous span information at a column position that was added with
+-- 'insertCurrentSpansAtColumn' if available.
+--
+-- If the previous 'RowSpan' @>= 1@, this will return 'Just' the previous
+-- 'ColSpan' and an adjusted span information where that 'RowSpan' is either
+-- decremented or deleted if it would fall to 0. Otherwise this will return
+-- 'Nothing'.
+takePreviousSpansAtColumn :: Int -> M.Map Int (RowSpan, ColSpan) -> Maybe (ColSpan, M.Map Int (RowSpan, ColSpan))
+takePreviousSpansAtColumn columnPosition previousSpans
+  | Just previous@(RowSpan previousRowSpan, previousColSpan) <- M.lookup columnPosition previousSpans
+  , previousRowSpan >= 1 = Just (previousColSpan, decrementPreviousRowSpans previous)
+  | otherwise = Nothing
+ where
+  decrementPreviousRowSpans (RowSpan previousRowSpan, previousColSpan) =
+    if previousRowSpan > 1
+      then M.insert columnPosition (RowSpan previousRowSpan - 1, previousColSpan) previousSpans
+      else M.delete columnPosition previousSpans
+
+-- | Decrement all previously tracked trailing 'RowSpan' elements at or after a
+-- column position.
+--
+-- For handling previous row spans that are next to the end of a row's cells
+-- that were previously added with 'insertCurrentSpansAtColumn'.
+decrementTrailingRowSpans :: Int -> M.Map Int (RowSpan, ColSpan) -> M.Map Int (RowSpan, ColSpan)
+decrementTrailingRowSpans columnPosition = M.mapWithKey decrementTrailing
+  where
+    decrementTrailing previousColumnPosition previousSpan@(RowSpan rowSpan, colSpan) =
+      if previousColumnPosition >= columnPosition && rowSpan >= 1
+        then (RowSpan rowSpan - 1, colSpan)
+        else previousSpan
