@@ -3,7 +3,7 @@
 {-# LANGUAGE FlexibleInstances    #-}
 {- |
    Module      : Tests.Helpers
-   Copyright   : © 2006-2022 John MacFarlane
+   Copyright   : © 2006-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley@edu>
@@ -16,6 +16,7 @@ module Tests.Helpers ( test
                      , TestResult(..)
                      , setupEnvironment
                      , showDiff
+                     , nativeDiff
                      , testGolden
                      , (=?>)
                      , purely
@@ -89,12 +90,14 @@ setupEnvironment testExePath = do
   mldpath   <- Env.lookupEnv "LD_LIBRARY_PATH"
   mdyldpath <- Env.lookupEnv "DYLD_LIBRARY_PATH"
   mpdd <- Env.lookupEnv "pandoc_datadir"
+  mbpath <- Env.lookupEnv "PATH"
   -- Note that Cabal sets the pandoc_datadir environment variable
   -- to point to the source directory, since otherwise getDataFilename
   -- will look in the data directory into which pandoc will be installed
   -- (but has not yet been).  So when we spawn a new process with
   -- pandoc, we need to make sure this environment variable is set.
-  return $ ("PATH",takeDirectory testExePath) :
+  return $ ("PATH",takeDirectory testExePath <>
+                    maybe mempty (searchPathSeparator:) mbpath) :
            ("TMP",".") :
            ("LANG","en_US.UTF-8") :
            ("HOME", "./") :
@@ -130,6 +133,19 @@ vividize (Both s _) = "  " ++ s
 vividize (First s)  = "- " ++ s
 vividize (Second s) = "+ " ++ s
 
+nativeDiff :: FilePath -> Pandoc -> Pandoc -> IO (Maybe String)
+nativeDiff normPath expectedNative actualNative
+  | expectedNative == actualNative = return Nothing
+  | otherwise = Just <$> do
+      expected <- T.unpack <$> runIOorExplode (writeNative def expectedNative)
+      actual <- T.unpack <$> runIOorExplode (writeNative def actualNative)
+      let dash = replicate 72 '-'
+      let diff = getDiff (lines actual) (lines expected)
+      return $ '\n' : dash ++
+               "\n--- " ++ normPath ++
+               "\n+++ " ++ "test" ++ "\n" ++
+               showDiff (1,1) diff ++ dash
+
 purely :: (b -> PandocPure a) -> b -> a
 purely f = either (error . show) id . runPure . f
 
@@ -155,7 +171,7 @@ instance ToString [Block] where
   toString = toString . B.fromList
 
 instance ToString Block where
-  toString = toString . B.singleton 
+  toString = toString . B.singleton
 
 instance ToString Inlines where
   toString = unpack . trimr . purely (writeNative def) . toPandoc
